@@ -13,13 +13,14 @@ package Lemonldap::NG::Manager;
 use 5.10.0;
 use utf8;
 use Mouse;
-our $VERSION = '1.9.18';
+use JSON;
 use Lemonldap::NG::Common::Conf::Constants;
 use Lemonldap::NG::Common::PSGI::Constants;
 
-extends 'Lemonldap::NG::Manager::Lib', 'Lemonldap::NG::Handler::PSGI::Router';
+our $VERSION = '1.9.99_02';
 
-has csp => ( is => 'rw' );
+extends 'Lemonldap::NG::Common::Conf::AccessLib',
+  'Lemonldap::NG::Handler::PSGI::Router';
 
 ## @method boolean init($args)
 # Launch initialization method
@@ -30,11 +31,11 @@ sub init {
     my ( $self, $args ) = @_;
     $args ||= {};
 
-    my $conf = $self->confAcc;
-
-    if ( my $localconf = $conf->getLocalConf(MANAGERSECTION) ) {
-        $self->{$_} = $args->{$_} // $localconf->{$_}
-          foreach ( keys %$localconf );
+    if ( my $localconf = $self->confAcc->getLocalConf(MANAGERSECTION) ) {
+        foreach ( keys %$localconf ) {
+            $args->{$_} //= $localconf->{$_};
+            $self->{$_} = $args->{$_} unless (/^(?:l|userL)ogger$/);
+        }
     }
 
     # Manager needs to keep new Ajax behaviour
@@ -56,27 +57,23 @@ sub init {
       split( /[,\s]+/, $self->{enabledModules} );
     extends 'Lemonldap::NG::Handler::PSGI::Router', @enabledModules;
     my @working;
+    my $conf = $self->confAcc->getConf;
     for ( my $i = 0 ; $i < @enabledModules ; $i++ ) {
         my $mod = $enabledModules[$i];
         no strict 'refs';
-        if ( &{"${mod}::addRoutes"}($self) ) {
-            $self->lmLog( "Module $mod enabled", 'debug' );
+        if ( &{"${mod}::addRoutes"}( $self, $conf ) ) {
+            $self->logger->debug("Module $mod enabled");
             push @working, $mod;
         }
         else {
             $links[$i] = undef;
-            $self->lmLog( "Module $mod can not be enabled: " . $self->error,
-                'error' );
+            $self->logger->error(
+                "Module $mod can not be enabled: " . $self->error );
         }
     }
     return 0 unless (@working);
-    $self->addRoute( links => 'links', ['GET'] );
-
-    my $portal = $Lemonldap::NG::Handler::Main::tsv->{portal}->();
-    $portal =~ s#https?://([^/]*).*#$1#;
-    $self->csp(
-"default-src 'self';frame-ancestors 'none';form-action 'self';img-src 'self' $portal;style-src 'self' $portal;"
-    );
+    $self->addRoute( links     => 'links',  ['GET'] );
+    $self->addRoute( 'psgi.js' => 'sendJs', ['GET'] );
 
     $self->defaultRoute( $working[0]->defaultRoute );
 
@@ -95,21 +92,19 @@ sub init {
     }
 
     $self->menuLinks( [] );
-    push @{ $self->menuLinks },
-      {
-        target => &{ $Lemonldap::NG::Handler::SharedConf::tsv->{portal} },
-        title  => 'backtoportal',
-        icon   => 'home'
-      }
-      if ( defined $Lemonldap::NG::Handler::SharedConf::tsv->{portal} );
-    push @{ $self->menuLinks },
-      {
-        target => &{ $Lemonldap::NG::Handler::SharedConf::tsv->{portal} }
-          . '?logout=1',
-        title => 'logout',
-        icon  => 'log-out'
-      }
-      if ( defined $Lemonldap::NG::Handler::SharedConf::tsv->{portal} );
+    if ( my $portal = Lemonldap::NG::Handler::PSGI::Main->tsv->{portal}->() ) {
+        push @{ $self->menuLinks },
+          {
+            target => $portal,
+            title  => 'backtoportal',
+            icon   => 'home'
+          },
+          {
+            target => "$portal?logout=1",
+            title  => 'logout',
+            icon   => 'log-out'
+          };
+    }
     1;
 }
 
@@ -121,26 +116,14 @@ sub javascript {
     my ($self) = @_;
     return
       'var formPrefix=staticPrefix+"forms/";var confPrefix=scriptname+"confs/";'
-      . (
-        $self->links ? 'var links=' . JSON::to_json( $self->links ) . ';' : '' )
+      . ( $self->links ? 'var links=' . to_json( $self->links ) . ';' : '' )
       . (
         $self->menuLinks
-        ? 'var menulinks=' . JSON::to_json( $self->menuLinks ) . ';'
+        ? 'var menulinks=' . to_json( $self->menuLinks ) . ';'
         : ''
       );
 }
 
-sub sendHtml {
-    my ( $self, $req, $template, %args ) = @_;
-    my $res = $self->SUPER::sendHtml( $req, $template, %args );
-    push @{ $res->[1] },
-      'Content-Security-Policy' => $self->csp,
-      'X-Content-Type-Options'  => 'nosniff',
-      'X-Frame-Options'         => 'DENY',
-      'X-XSS-Protection'        => '1; mode=block';
-    return $res;
-}
-
 1;
 __END__
 
@@ -153,7 +136,9 @@ system.
 
 =head1 SYNOPSIS
 
-  #!/usr/bin/env plackup -I pl/lib
+Use any of Plack launcher. Example:
+
+  #!/usr/bin/env plackup
   
   use Lemonldap::NG::Manager;
   
@@ -174,25 +159,19 @@ inherits of L<Lemonldap::NG::Handler::PSGI::Router>
 
 =head1 ORGANIZATION
 
-Lemonldap::NG Manager contains 4 parts:
+Lemonldap::NG Manager contains 5 parts:
 
 =over
 
-=item Configuration management:
+=item Configuration management
 
-see L<Lemonldap::NG::Manager::Conf>;
+=item Session explorer
 
-=item Session explorer:
+=item Notification explorer
 
-see L<Lemonldap::NG::Manager::Sessions>;
+=item Configuration builder (see L<Lemonldap::NG::Manager::Build>
 
-=item Notification explorer:
-
-see L<Lemonldap::NG::Manager::Notifications>;
-
-=item Some files uses to generate static files:
-
-see below.
+=item Command line (see L<Lemonldap::NG::Manager::Cli>
 
 =back
 
@@ -203,15 +182,15 @@ Lemonldap::NG::Manager::Build::Tree and Lemonldap::NG::Manager::Build::CTrees to
 
 =over
 
-=item `site/static/struct.json`:
+=item `site/htdocs/static/struct.json`:
 
 the main file that contains the tree view;
 
-=item `site/static/js/conftree.js`:
+=item `site/htdocs/static/js/conftree.js`:
 
 generates sub tree for virtualhosts and SAML and OpenID-Connect partners;
 
-=item `Lemonldap::NG::Manager::Constants`:
+=item `Lemonldap::NG::Common::Conf::ReConstants`:
 
 constants used by all Perl manager components;
 
@@ -259,29 +238,23 @@ you can also fix them in $opts hash ref passed as argument to run() or new()).
 
 =head1 SEE ALSO
 
-L<Lemonldap::NG::Handler::Router>, L<Lemonldap::NG::Portal>, L<Plack>, L<PSGI>,
-L<Lemonldap::NG::Manager::Conf>, L<Lemonldap::NG::Manager::Sessions>,
-L<Lemonldap::NG::Manager::Notifications>
 L<http://lemonldap-ng.org/>
 
 =head1 AUTHORS
 
 =over
 
-=item Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
-
-=item François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
-
-=item Xavier Guimard, E<lt>x.guimard@free.frE<gt>
-
-=item Thomas Chemineau, E<lt>thomas.chemineau@gmail.comE<gt>
+=item LemonLDAP::NG team L<http://lemonldap-ng.org/team>
 
 =back
 
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
+L<http://jira.ow2.org>
+
+Note that if you want to post a ticket for a conf upload problem, please
+see L<Lemonldap::NG::Manager::Conf::Parser> before.
 
 =head1 DOWNLOAD
 
@@ -290,13 +263,7 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =head1 COPYRIGHT AND LICENSE
 
-=over
-
-=item Copyright (C) 2015-2016 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
-
-=item Copyright (C) 2015-2016 by Clément Oudot, E<lt>clem.oudot@gmail.comE<gt>
-
-=back
+See COPYING file for details.
 
 This library is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
