@@ -6,28 +6,18 @@
 
 package Lemonldap::NG::Manager::Build::Attributes;
 
-our $VERSION = '1.9.11';
+our $VERSION = '1.9.99_02';
 use strict;
 use Regexp::Common qw/URI/;
 
 my $perlExpr = sub {
     my ( $val, $conf ) = @_;
-    my $s  = '';
-    my @cf = qw(
-      encode_base64 checkLogonHours date checkDate basic unicode2iso
-      iso2unicode groupMatch encrypt
-    );
-    push @cf,
-      defined $conf->{customFunctions}
-      ? map { my $f = $_; $f =~ s/\w+:://g; ( $f, $_ ) }
-      split( /\s+/, $conf->{customFunctions} )
-      : ();
-    foreach my $f (@cf) {
-        $s = "sub $f {1} $s";
-    }
+    my $s = '';
     no warnings( 'redefine', 'uninitialized' );
     eval "$s $val";
-    return $@ ? ( 1, "__badExpression__: $@" ) : (1);
+    my $err = join( '',
+        grep { $_ =~ /Undefined subroutine/ ? () : $_ } split( /\n/, $@ ) );
+    return $err ? ( 1, "__badExpression__: $err" ) : (1);
 };
 
 my $url = $RE{URI}{HTTP}{ -scheme => "https?" };
@@ -120,7 +110,7 @@ sub types {
                     { $_ eq $_[0] } map ( { $_->{k} } @{ $_[2]->{select} } ) );
                 return $test
                   ? 1
-                  : ( 0, "Invalid value '$_[0]' for this select" );
+                  : ( 1, "Invalid value '$_[0]' for this select" );
             },
         },
 
@@ -152,7 +142,7 @@ sub types {
             test => sub {
                 return (
                     $_[0] =~
-/^(?:(?:\-+\s*BEGIN\s+(?:RSA\s+)?PRIVATE\s+KEY\s*\-+\r?\n)?(?:Proc-Type:.*\r?\nDEK-Info:.*\r?\n[\r\n]*)?[a-zA-Z0-9\/\+\r\n]+={0,2}(?:\r?\n\-+\s*END\s+(?:RSA\s+)PRIVATE\s+KEY\s*\-+)?[\r\n]*)?$/s
+/^(?:(?:\-+\s*BEGIN\s+(?:RSA\s+)?PRIVATE\s+KEY\s*\-+\r?\n)?[a-zA-Z0-9\/\+\r\n]+={0,2}(?:\r?\n\-+\s*END\s+(?:RSA\s+)PRIVATE\s+KEY\s*\-+)?[\r\n]*)?$/s
                     ? (1)
                     : ( 1, '__badPemEncoding__' )
                 );
@@ -304,9 +294,15 @@ sub attributes {
             default       => 'http://auth.example.com/',
             documentation => 'Portal URL',
         },
+        portalStatus => {
+            type          => 'bool',
+            default       => 0,
+            documentation => 'Enable portal status',
+        },
         portalUserAttr => {
             type    => 'text',
             default => '_user',
+            help    => 'monitoring.html',
             documentation =>
               'Session parameter to display connected user in portal',
         },
@@ -328,13 +324,10 @@ sub attributes {
             type          => 'text',
             documentation => 'Prefix of static files for HTML templates',
         },
-        syslog => {
-            type => 'text',
-            test => qr/^(?:auth|authpriv|daemon|local\d|user)?$/,
-            msgFail =>
-              '__authorizedValues__: auth, authpriv, daemon, local0-7, user',
-            default       => '',
-            documentation => 'Syslog facility',
+        multiValuesSeparator => {
+            type          => 'authParamsText',
+            default       => '; ',
+            documentation => 'Separator for multiple values',
         },
 
         # Manager
@@ -348,6 +341,7 @@ sub attributes {
 
         # Menu
         activeTimer => {
+            default       => 0,
             type          => 'bool',
             default       => 1,
             documentation => 'Enable timers on portal pages',
@@ -361,17 +355,6 @@ sub attributes {
             },
             documentation => 'Applications list',
         },
-        portalErrorOnExpiredSession => {
-            type          => 'bool',
-            default       => 1,
-            documentation => 'Show error if session is expired',
-        },
-        portalErrorOnMailNotFound => {
-            type    => 'bool',
-            default => 0,
-            documentation =>
-              'Show error if mail is not found in password reset process',
-        },
         portalOpenLinkInNewWindow => {
             type          => 'bool',
             default       => 0,
@@ -429,6 +412,16 @@ sub attributes {
         },
 
         # Security
+        formTimeout => {
+            default       => 120,
+            type          => 'int',
+            documentation => 'Token timeout for forms',
+        },
+        requireToken => {
+            default       => 1,
+            type          => 'bool',
+            documentation => 'Enable token for forms',
+        },
         cda => {
             default       => 0,
             type          => 'bool',
@@ -453,11 +446,43 @@ sub attributes {
             type          => 'password',
             documentation => 'Secret key',
         },
+        cspDefault => {
+            type          => 'text',
+            default       => "'self'",
+            documentation => 'Default value for Content-Security-Policy',
+        },
+        cspImg => {
+            type          => 'text',
+            default       => "'self'",
+            documentation => 'Image source for Content-Security-Policy',
+        },
+        cspScript => {
+            type          => 'text',
+            default       => "'self'",
+            documentation => 'Javascript source for Content-Security-Policy',
+        },
+        cspStyle => {
+            type          => 'text',
+            default       => "'self'",
+            documentation => 'Style source for Content-Security-Policy',
+        },
+        cspConnect => {
+            type    => 'text',
+            default => "'self'",
+            documentation =>
+              'Authorizated Ajax destination for Content-Security-Policy',
+        },
+        cspFont => {
+            type          => 'text',
+            default       => "'self'",
+            documentation => 'Font source for Content-Security-Policy',
+        },
         portalAntiFrame => {
             default       => 1,
             type          => 'bool',
             documentation => 'Avoid portal to be displayed inside frames',
         },
+
         portalCheckLogins => {
             default       => 1,
             type          => 'bool',
@@ -526,6 +551,7 @@ sub attributes {
         useSafeJail => {
             default       => 1,
             type          => 'bool',
+            help          => 'safejail.html',
             documentation => 'Activate Safe jail',
         },
         whatToTrace => {
@@ -533,10 +559,14 @@ sub attributes {
             default       => 'uid',
             documentation => 'Session parameter used to fill REMOTE_USER',
         },
-        lwpSslOpts => {
+        lwpOpts => {
             type          => 'keyTextContainer',
             documentation => 'Options given to LWP::UserAgent',
         },
+        lwpSslOpts => {
+            type          => 'keyTextContainer',
+            documentation => 'SSL options given to LWP::UserAgent',
+        },
 
         # History
         failedLoginNumber => {
@@ -547,7 +577,6 @@ sub attributes {
         loginHistoryEnabled => {
             default       => 0,
             type          => 'bool',
-            default       => 1,
             documentation => 'Enable login history',
         },
         portalDisplayLoginHistory => {
@@ -599,7 +628,7 @@ sub attributes {
         },
         domain => {
             type          => 'text',
-            test          => qr/^(?:$Regexp::Common::URI::RFC2396::hostname)?/,
+            test          => qr/^(?:$Regexp::Common::URI::RFC2396::hostname)?$/,
             msgFail       => '__badDomainName__',
             default       => 'example.com',
             documentation => 'DNS domain',
@@ -622,6 +651,11 @@ sub attributes {
         },
 
         # Notification
+        oldNotifFormat => {
+            type          => 'bool',
+            default       => 0,
+            documentation => 'Use old XML format',
+        },
         notificationWildcard => {
             type          => 'text',
             default       => 'allusers',
@@ -633,6 +667,11 @@ sub attributes {
             type          => 'bool',
             documentation => 'Notification activation',
         },
+        notificationServer => {
+            default       => 0,
+            type          => 'bool',
+            documentation => 'Notification server activation',
+        },
         notificationStorage => {
             type          => 'PerlModule',
             default       => 'File',
@@ -666,19 +705,6 @@ sub attributes {
             documentation => 'Captcha size',
         },
 
-        #captcha_data
-        #captcha_output
-        captchaStorage => {
-            type          => 'PerlModule',
-            default       => 'Apache::Session::File',
-            documentation => 'Captcha backend module',
-        },
-        captchaStorageOptions => {
-            type    => 'keyTextContainer',
-            default => { 'Directory' => '/var/lib/lemonldap-ng/captcha/', },
-            documentation => 'Captcha backend module options',
-        },
-
         # Variables
         exportedVars => {
             type          => 'keyTextContainer',
@@ -771,7 +797,7 @@ sub attributes {
         # OpenID-Connect issuer
         issuerDBOpenIDConnectActivation => {
             type          => 'bool',
-            default       => 0,
+            default       => '0',
             documentation => 'OpenID Connect server activation',
         },
         issuerDBOpenIDConnectPath => {
@@ -788,7 +814,7 @@ sub attributes {
         # GET issuer
         issuerDBGetActivation => {
             type          => 'bool',
-            default       => 0,
+            default       => '0',
             documentation => 'Get issuer activation',
         },
         issuerDBGetPath => {
@@ -852,7 +878,6 @@ sub attributes {
         mailConfirmBody    => { type => 'longtext', },
         mailConfirmSubject => {
             type          => 'text',
-            default       => '[LemonLDAP::NG] Password reset confirmation',
             documentation => 'Mail subject for reset confirmation',
         },
         mailFrom => {
@@ -868,7 +893,6 @@ sub attributes {
         },
         mailSubject => {
             type          => 'text',
-            default       => '[LemonLDAP::NG] Your new password',
             documentation => 'Mail subject for new password email',
         },
         mailTimeout => {
@@ -878,7 +902,7 @@ sub attributes {
         },
         mailUrl => {
             type          => 'url',
-            default       => 'http://auth.example.com/mail.pl',
+            default       => 'http://auth.example.com/resetpwd',
             documentation => 'URL of password reset page',
         },
         SMTPServer => {
@@ -893,7 +917,6 @@ sub attributes {
         # Registration
         registerConfirmSubject => {
             type          => 'text',
-            default       => '[LemonLDAP::NG] Account register confirmation',
             documentation => 'Mail subject for register confirmation',
         },
         registerDB => {
@@ -904,12 +927,11 @@ sub attributes {
                 { k => 'LDAP', v => 'LDAP' },
                 { k => 'Null', v => 'None' },
             ],
-            default       => 'Demo',
+            default       => 'Null',
             documentation => 'Register module',
         },
         registerDoneSubject => {
             type          => 'text',
-            default       => '[LemonLDAP::NG] Your new account',
             documentation => 'Mail subject when register is done',
         },
         registerTimeout => {
@@ -917,10 +939,22 @@ sub attributes {
             type          => 'int',
             documentation => 'Register session timeout',
         },
-        registerUrl => {
-            type          => 'text',
-            default       => 'http://auth.example.com/register.pl',
-            documentation => 'URL of register page',
+
+        # U2F
+        u2fActivation => {
+            type          => 'bool',
+            default       => 0,
+            documentation => 'U2F activation',
+        },
+        u2fSelfRegistration => {
+            type          => 'bool',
+            default       => 0,
+            documentation => 'U2F self registration activation',
+        },
+        u2fAuthnLevel => {
+            type => 'int',
+            documentation =>
+              'Authentication level for users authentified by password+U2F'
         },
 
         # Single session
@@ -954,13 +988,36 @@ sub attributes {
             documentation => 'Allow only one session per user on an IP',
         },
 
-        # SOAP server
-        Soap => {
+        # REST server
+        restSessionServer => {
             default       => 0,
             type          => 'bool',
-            documentation => 'Enable SOAP services',
+            documentation => 'Enable REST session server',
+        },
+        restConfigServer => {
+            default       => 0,
+            type          => 'bool',
+            documentation => 'Enable REST config server',
+        },
+
+        # SOAP server
+        soapSessionServer => {
+            default       => 0,
+            type          => 'bool',
+            help          => 'soapservices.html',
+            documentation => 'Enable SOAP session server',
+        },
+        soapConfigServer => {
+            default       => 0,
+            type          => 'bool',
+            help          => 'soapservices.html',
+            documentation => 'Enable SOAP config server',
+        },
+        exportedAttr => {
+            type => 'text',
+            documentation =>
+              'List of attributes to export by SOAP or REST servers',
         },
-        exportedAttr => { type => 'text', },
 
         ## Virtualhosts
 
@@ -990,19 +1047,12 @@ sub attributes {
                           : ( 0, '__badUrl__' );
                     }
                     $s =~ s/\b(accept|deny|unprotect|skip)\b/1/g;
-                    my @cf =
-                      qw(encode_base64 checkLogonHours date checkDate basic unicode2iso iso2unicode groupMatch encrypt);
-                    push @cf,
-                      defined $conf->{customFunctions}
-                      ? map { my $f = $_; $f =~ s/\w+:://g; ( $f, $_ ) }
-                      split( /\s+/, $conf->{customFunctions} )
-                      : ();
-                    foreach my $f (@cf) {
-                        $s = "sub $f {1} $s";
-                    }
                     no warnings( 'redefine', 'uninitialized' );
                     eval $s;
-                    return $@ ? ( 1, "__badExpression__: $@" ) : (1);
+                    my $err = join( '',
+                        grep { $_ =~ /Undefined subroutine/ ? () : $_ }
+                          split( /\n/, $@ ) );
+                    return $err ? ( 1, "__badExpression__: $err" ) : (1);
                 },
                 msgFail => '__badExpression__',
             },
@@ -1024,19 +1074,12 @@ sub attributes {
                 test       => sub {
                     my ( $val, $conf ) = @_;
                     my $s = $val;
-                    my @cf =
-                      qw(encode_base64 checkLogonHours date checkDate basic unicode2iso iso2unicode groupMatch encrypt);
-                    push @cf,
-                      defined $conf->{customFunctions}
-                      ? map { my $f = $_; $f =~ s/\w+:://g; ( $f, $_ ) }
-                      split( /\s+/, $conf->{customFunctions} )
-                      : ();
-                    foreach my $f (@cf) {
-                        $s = "sub $f {1} $s";
-                    }
                     no warnings( 'redefine', 'uninitialized' );
                     eval $s;
-                    return $@ ? ( 1, "__badExpression__: $@" ) : (1);
+                    my $err = join( '',
+                        grep { $_ =~ /Undefined subroutine/ ? () : $_ }
+                          split( /\n/, $@ ) );
+                    return $err ? ( 1, "__badExpression__: $err" ) : (1);
                   }
             },
             documentation => 'Virtualhost headers',
@@ -1068,6 +1111,20 @@ sub attributes {
         vhostAliases => {
             type => 'text',
         },
+        vhostType => {
+            type   => 'select',
+            select => [
+                { k => 'Main',         v => 'Main' },
+                { k => 'Zimbra',       v => 'ZimbraPreAuth' },
+                { k => 'AuthBasic',    v => 'AuthBasic' },
+                { k => 'SecureToken',  v => 'SecureToken' },
+                { k => 'CDA',          v => 'CDA' },
+                { k => 'DevOps',       v => 'DevOps' },
+                { k => 'ServiceToken', v => 'ServiceToken' },
+            ],
+            default       => 'Main',
+            documentation => 'Handler type',
+        },
 
         # CAS IDP
         casAttr                => { type => 'text', },
@@ -1148,58 +1205,6 @@ sub attributes {
         openIdSreg_dob => { type => 'lmAttrOrMacro', },
         openIdSPList   => { type => 'blackWhiteList', default => '0;' },
 
-        # Zimbra
-        zimbraPreAuthKey => { type => 'text', },
-        zimbraAccountKey => { type => 'text', },
-        zimbraBy         => {
-            type   => 'select',
-            select => [
-                { k => '',                 v => '' },
-                { k => 'name',             v => 'User name' },
-                { k => 'id',               v => 'User id' },
-                { k => 'foreignPrincipal', v => 'Foreign principal' },
-            ],
-            default => '',
-        },
-        zimbraUrl    => { type => 'text', },
-        zimbraSsoUrl => { type => 'text', },
-
-        # Sympa
-        sympaSecret  => { type => 'text', },
-        sympaMailKey => { type => 'text', },
-
-        # Secure Token
-        secureTokenMemcachedServers => {
-            type          => 'text',
-            default       => '127.0.0.1:11211',
-            documentation => 'Secure Token Handler memcached servers',
-        },
-        secureTokenExpiration => {
-            type          => 'int',
-            default       => 60,
-            documentation => 'Secure Token Handler token expiration',
-        },
-        secureTokenAttribute => {
-            type          => 'text',
-            default       => 'uid',
-            documentation => 'Secure Token Handler attribute to store',
-        },
-        secureTokenUrls => {
-            type    => 'pcre',
-            default => '.*',
-            documentation =>
-              'Secure Token Handler regular expression to match protected URL',
-        },
-        secureTokenHeader => {
-            type          => 'text',
-            default       => 'Auth-Token',
-            documentation => 'Secure Token Handler header name',
-        },
-        secureTokenAllowOnError => {
-            default       => 1,
-            type          => 'bool',
-            documentation => 'Secure Token Handler allow request on error',
-        },
         #########
         ## SAML #
         #########
@@ -1367,12 +1372,6 @@ sub attributes {
               . '#PORTAL#/saml/singleSignOnArtifact;',
             documentation => 'SAML IDP SSO HTTP Artifact',
         },
-        samlIDPSSODescriptorSingleSignOnServiceSOAP => {
-            type    => 'samlService',
-            default => 'urn:oasis:names:tc:SAML:2.0:bindings:SOAP;'
-              . '#PORTAL#/saml/singleSignOnSOAP;',
-            documentation => 'SAML IDP SSO SOAP',
-        },
         samlIDPSSODescriptorSingleLogoutServiceHTTPRedirect => {
             type    => 'samlService',
             default => 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect;'
@@ -1428,6 +1427,31 @@ sub attributes {
         },
         samlIDPMetaDataXML => {
             type => 'file',
+            test => sub {
+                my $v = shift;
+                return 1 unless ( $v and %$v );
+                my @msg;
+                my $res = 1;
+                my %entityIds;
+                foreach my $idpId ( keys %$v ) {
+                    unless ( $v->{$idpId}->{samlIDPMetaDataXML} =~
+                        /entityID="(.+?)"/si )
+                    {
+                        push @msg, "$idpId SAML metadata has ne EntityID";
+                        $res = 0;
+                        next;
+                    }
+                    my $eid = $1;
+                    if ( defined $entityIds{$eid} ) {
+                        push @msg,
+"$idpId and $entityIds{$eid} have the same SAML EntityID";
+                        $res = 0;
+                        next;
+                    }
+                    $entityIds{$eid} = $idpId;
+                }
+                return ( $res, join( ', ', @msg ) );
+            },
         },
         samlIDPMetaDataOptions => {
             type       => 'keyTextContainer',
@@ -1510,9 +1534,7 @@ sub attributes {
                 { k => '',              v => '' },
                 { k => 'http-post',     v => 'POST' },
                 { k => 'http-redirect', v => 'Redirect' },
-                { k => 'http-soap',     v => 'SOAP' },
                 { k => 'artifact-get',  v => 'Artifact GET' },
-                { k => 'artifact-post', v => 'Artifact POST' },
             ],
             default => '',
         },
@@ -1523,8 +1545,6 @@ sub attributes {
                 { k => 'http-post',     v => 'POST' },
                 { k => 'http-redirect', v => 'Redirect' },
                 { k => 'http-soap',     v => 'SOAP' },
-                { k => 'artifact-get',  v => 'Artifact GET' },
-                { k => 'artifact-post', v => 'Artifact POST' },
             ],
             default => '',
         },
@@ -1694,31 +1714,29 @@ sub attributes {
         authentication => {
             type   => 'select',
             select => [
-                { k => 'Apache',    v => 'Apache' },
-                { k => 'AD',        v => 'Active Directory' },
-                { k => 'BrowserID', v => 'BrowserID (Mozilla Persona)' },
-                { k => 'Choice',    v => 'authChoice' },
-                { k => 'CAS',  v => 'Central Authentication Service (CAS)' },
-                { k => 'DBI',  v => 'Database (DBI)' },
-                { k => 'Demo', v => 'Demonstration' },
-                { k => 'Facebook',      v => 'Facebook' },
-                { k => 'Google',        v => 'Google' },
-                { k => 'Kerberos',      v => 'Kerberos' },
-                { k => 'LDAP',          v => 'LDAP' },
-                { k => 'LinkedIn',      v => 'LinkedIn' },
-                { k => 'Multi',         v => 'Multiple' },
-                { k => 'Null',          v => 'None' },
-                { k => 'OpenID',        v => 'OpenID' },
+                { k => 'Apache',      v => 'Apache' },
+                { k => 'AD',          v => 'Active Directory' },
+                { k => 'DBI',         v => 'Database (DBI)' },
+                { k => 'Facebook',    v => 'Facebook' },
+                { k => 'Google',      v => 'Google' },
+                { k => 'LDAP',        v => 'LDAP' },
+                { k => 'Radius',      v => 'Radius' },
+                { k => 'REST',        v => 'REST' },
+                { k => 'SSL',         v => 'SSL' },
+                { k => 'Twitter',     v => 'Twitter' },
+                { k => 'WebID',       v => 'WebID' },
+                { k => 'Yubikey',     v => 'Yubikey' },
+                { k => 'Demo',        v => 'Demonstration' },
+                { k => 'Choice',      v => 'authChoice' },
+                { k => 'Combination', v => 'combineMods' },
+                { k => 'CAS',    v => 'Central Authentication Service (CAS)' },
+                { k => 'OpenID', v => 'OpenID' },
                 { k => 'OpenIDConnect', v => 'OpenID Connect' },
-                { k => 'Proxy',         v => 'Proxy' },
-                { k => 'Radius',        v => 'Radius' },
-                { k => 'Remote',        v => 'Remote' },
                 { k => 'SAML',          v => 'SAML v2' },
+                { k => 'Proxy',         v => 'Proxy' },
+                { k => 'Remote',        v => 'Remote' },
                 { k => 'Slave',         v => 'Slave' },
-                { k => 'SSL',           v => 'SSL' },
-                { k => 'Twitter',       v => 'Twitter' },
-                { k => 'WebID',         v => 'WebID' },
-                { k => 'Yubikey',       v => 'Yubikey' },
+                { k => 'Null',          v => 'None' },
             ],
             default       => 'Demo',
             documentation => 'Authentication module',
@@ -1726,24 +1744,14 @@ sub attributes {
         userDB => {
             type   => 'select',
             select => [
-                { k => 'AD',            v => 'Active Directory' },
-                { k => 'DBI',           v => 'Database (DBI)' },
-                { k => 'Choice',        v => 'authChoice' },
-                { k => 'Demo',          v => 'Demonstration' },
-                { k => 'Facebook',      v => 'Facebook' },
-                { k => 'Google',        v => 'Google' },
-                { k => 'LDAP',          v => 'LDAP' },
-                { k => 'Multi',         v => 'Multiple' },
-                { k => 'Null',          v => 'None' },
-                { k => 'OpenID',        v => 'OpenID' },
-                { k => 'OpenIDConnect', v => 'OpenID Connect' },
-                { k => 'Proxy',         v => 'Proxy' },
-                { k => 'Remote',        v => 'Remote' },
-                { k => 'SAML',          v => 'SAML v2' },
-                { k => 'Slave',         v => 'Slave' },
-                { k => 'WebID',         v => 'WebID' },
+                { k => 'Same', v => 'Same' },
+                { k => 'AD',   v => 'Active Directory' },
+                { k => 'DBI',  v => 'Database (DBI)' },
+                { k => 'LDAP', v => 'LDAP' },
+                { k => 'REST', v => 'REST' },
+                { k => 'Null', v => 'None' },
             ],
-            default       => 'Demo',
+            default       => 'Same',
             documentation => 'User module',
         },
         passwordDB => {
@@ -1754,6 +1762,7 @@ sub attributes {
                 { k => 'DBI',    v => 'Database (DBI)' },
                 { k => 'Demo',   v => 'Demonstration' },
                 { k => 'LDAP',   v => 'LDAP' },
+                { k => 'REST',   v => 'REST' },
                 { k => 'Null',   v => 'None' },
             ],
             default       => 'Demo',
@@ -1944,7 +1953,8 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
             default       => 5,
             documentation => 'SSL authentication level',
         },
-        SSLVar => { type => 'text', },
+        SSLVar   => { type => 'text', },
+        SSLVarIf => { type => 'keyTextContainer', },
 
         # CAS
         CAS_authnLevel => {
@@ -1980,6 +1990,12 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         radiusSecret => { type => 'text', },
         radiusServer => { type => 'text', },
 
+        # REST
+        restAuthUrl       => { type => 'url' },
+        restUserDBUrl     => { type => 'url' },
+        restPwdConfirmUrl => { type => 'url' },
+        restPwdModifyUrl  => { type => 'url' },
+
         # Remote
         remotePortal        => { type => 'text', },
         remoteGlobalStorage => {
@@ -1990,17 +2006,27 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         remoteGlobalStorageOptions => {
             type    => 'keyTextContainer',
             default => {
-                proxy => 'http://auth.example.com/index.pl/sessions',
+                proxy => 'http://auth.example.com/sessions',
                 ns =>
-                  'http://auth.example.com/Lemonldap/NG/Common/CGI/SOAPService',
+'http://auth.example.com/Lemonldap/NG/Common/PSGI/SOAPService',
             },
             documentation => 'Demo exported variables',
         },
 
         # Proxy
-        soapAuthService    => { type => 'text', },
-        remoteCookieName   => { type => 'text', },
-        soapSessionService => { type => 'text', },
+        proxyAuthService    => { type => 'text', },
+        proxySessionService => { type => 'text', },
+        remoteCookieName    => { type => 'text', },
+        proxyUseSoap        => {
+            type          => 'bool',
+            default       => 0,
+            documentation => 'Use SOAP instead of REST',
+        },
+        proxyAuthnLevel => {
+            type          => 'int',
+            default       => 2,
+            documentation => 'Proxy authentication level',
+        },
 
         # OpenID
         openIdAuthnLevel => {
@@ -2020,22 +2046,6 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         },
         'openIdIDPList' => { 'type' => 'blackWhiteList', default => '0;' },
 
-        # Google
-        googleAuthnLevel => {
-            type          => 'int',
-            default       => 1,
-            documentation => 'Google authentication level',
-        },
-        googleExportedVars => {
-            type          => 'keyTextContainer',
-            keyTest       => qr/^!?[a-zA-Z][a-zA-Z0-9_-]*$/,
-            keyMsgFail    => '__badVariableName__',
-            test          => qr/^[a-zA-Z][a-zA-Z0-9_:\-]*$/,
-            msgFail       => '__badValue__',
-            default       => {},
-            documentation => 'Google exported variables',
-        },
-
         # Facebook
         facebookAuthnLevel => {
             type          => 'int',
@@ -2064,22 +2074,6 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         twitterSecret  => { type => 'text', },
         twitterAppName => { type => 'text', },
 
-        # LinkedIn
-        linkedInAuthnLevel => {
-            type          => 'int',
-            default       => 1,
-            documentation => 'LinkedIn authentication level',
-        },
-        linkedInClientID     => { type => 'text', },
-        linkedInClientSecret => { type => 'password', },
-        linkedInFields       => {
-            type    => 'text',
-            default => 'id,first-name,last-name,email-address'
-        },
-        linkedInUserField => { type => 'text', default => 'emailAddress' },
-        linkedInScope =>
-          { type => 'text', default => 'r_basicprofile r_emailaddress' },
-
         # WebID
         webIDAuthnLevel => {
             type          => 'int',
@@ -2117,14 +2111,6 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         userPivot          => { type => 'text', },
         dbiAuthPasswordHash =>
           { type => 'text', help => 'authdbi.html#password', },
-        dbiDynamicHashEnabled =>
-          { type => 'bool', help => 'authdbi.html#password', },
-        dbiDynamicHashValidSchemes =>
-          { type => 'text', help => 'authdbi.html#password', },
-        dbiDynamicHashValidSaltedSchemes =>
-          { type => 'text', help => 'authdbi.html#password', },
-        dbiDynamicHashNewPasswordScheme =>
-          { type => 'text', help => 'authdbi.html#password', },
         dbiExportedVars => {
             type          => 'keyTextContainer',
             keyTest       => qr/^!?[a-zA-Z][a-zA-Z0-9_-]*$/,
@@ -2149,32 +2135,6 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
             documentation => 'Null authentication level',
         },
 
-        # Kerberos
-        krbKeytab => {
-            type          => 'text',
-            documentation => 'Kerberos keytab',
-        },
-        krbByJs => {
-            type          => 'bool',
-            default       => 0,
-            documentation => 'Launch Kerberos authentication by Ajax',
-        },
-        krbAuthnLevel => {
-            type          => 'int',
-            default       => 3,
-            documentation => 'Null authentication level',
-        },
-        krbUseModKrb => {
-            type          => 'bool',
-            default       => 0,
-            documentation => 'Rely on Web Server Kerberos module',
-        },
-        krbRemoveDomain => {
-            type          => 'bool',
-            default       => 1,
-            documentation => 'Remove domain in Kerberos username',
-        },
-
         # Slave
         slaveAuthnLevel => {
             type          => 'int',
@@ -2212,22 +2172,20 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
             test       => sub { 1 },
             select     => [
                 [
-                    { k => 'Apache',    v => 'Apache' },
-                    { k => 'AD',        v => 'Active Directory' },
-                    { k => 'BrowserID', v => 'BrowserID (Mozilla Persona)' },
+                    { k => 'Apache', v => 'Apache' },
+                    { k => 'AD',     v => 'Active Directory' },
                     { k => 'CAS', v => 'Central Authentication Service (CAS)' },
                     { k => 'DBI', v => 'Database (DBI)' },
                     { k => 'Demo',          v => 'Demo' },
                     { k => 'Facebook',      v => 'Facebook' },
                     { k => 'Google',        v => 'Google' },
-                    { k => 'Kerberos',      v => 'Kerberos' },
                     { k => 'LDAP',          v => 'LDAP' },
-                    { k => 'LinkedIn',      v => 'LinkedIn' },
                     { k => 'Null',          v => 'None' },
                     { k => 'OpenID',        v => 'OpenID' },
                     { k => 'OpenIDConnect', v => 'OpenID Connect' },
                     { k => 'Proxy',         v => 'Proxy' },
                     { k => 'Radius',        v => 'Radius' },
+                    { k => 'REST',          v => 'REST' },
                     { k => 'Remote',        v => 'Remote' },
                     { k => 'SAML',          v => 'SAML v2' },
                     { k => 'Slave',         v => 'Slave' },
@@ -2243,6 +2201,7 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
                     { k => 'Facebook',      v => 'Facebook' },
                     { k => 'Google',        v => 'Google' },
                     { k => 'LDAP',          v => 'LDAP' },
+                    { k => 'REST',          v => 'REST' },
                     { k => 'Null',          v => 'None' },
                     { k => 'OpenID',        v => 'OpenID' },
                     { k => 'OpenIDConnect', v => 'OpenID Connect' },
@@ -2257,22 +2216,45 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
                     { k => 'DBI',  v => 'Database (DBI)' },
                     { k => 'Demo', v => 'Demo' },
                     { k => 'LDAP', v => 'LDAP' },
+                    { k => 'REST', v => 'REST' },
                     { k => 'Null', v => 'None' }
                 ]
             ],
         },
 
-        # Multi
-        multiAuthStack => {
-            type => 'authParamsText',
+        # Combination
+        combination => {
+            type          => 'text',
+            documentation => 'Combination rule'
         },
-        multiUserDBStack => {
-            type => 'authParamsText',
-        },
-        multiValuesSeparator => {
-            type          => 'authParamsText',
-            default       => '; ',
-            documentation => 'Separator for multiple values',
+        combModules => {
+            type          => 'cmbModuleContainer',
+            keyTest       => qr/^\w+$/,
+            test          => sub { 1 },
+            documentation => 'Combination module description',
+            select        => [
+                { k => 'Apache',   v => 'Apache' },
+                { k => 'AD',       v => 'Active Directory' },
+                { k => 'DBI',      v => 'Database (DBI)' },
+                { k => 'Facebook', v => 'Facebook' },
+                { k => 'Google',   v => 'Google' },
+                { k => 'LDAP',     v => 'LDAP' },
+                { k => 'Radius',   v => 'Radius' },
+                { k => 'REST',     v => 'REST' },
+                { k => 'SSL',      v => 'SSL' },
+                { k => 'Twitter',  v => 'Twitter' },
+                { k => 'WebID',    v => 'WebID' },
+                { k => 'Yubikey',  v => 'Yubikey' },
+                { k => 'Demo',     v => 'Demonstration' },
+                { k => 'CAS',    v => 'Central Authentication Service (CAS)' },
+                { k => 'OpenID', v => 'OpenID' },
+                { k => 'OpenIDConnect', v => 'OpenID Connect' },
+                { k => 'SAML',          v => 'SAML v2' },
+                { k => 'Proxy',         v => 'Proxy' },
+                { k => 'Remote',        v => 'Remote' },
+                { k => 'Slave',         v => 'Slave' },
+                { k => 'Null',          v => 'None' },
+            ],
         },
 
         # Yubikey
@@ -2289,18 +2271,6 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
             documentation => 'Yubikey public ID size',
         },
 
-        # BrowserID
-        browserIdAuthnLevel => {
-            type          => 'int',
-            default       => 1,
-            documentation => 'Browser ID authentication level',
-        },
-        browserIdAutoLogin       => { type => 'bool', },
-        browserIdVerificationURL => { type => 'text', },
-        browserIdSiteName        => { type => 'text', },
-        browserIdSiteLogo        => { type => 'text', },
-        browserIdBackgroundColor => { type => 'text', },
-
         # OpenID Connect auth params
         oidcAuthnLevel => {
             type          => 'int',
@@ -2356,7 +2326,7 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         },
         oidcServiceMetaDataCheckSessionURI => {
             type          => 'text',
-            default       => 'checksession',
+            default       => 'checksession.html',
             documentation => 'OpenID Connect check session iframe',
         },
         oidcServiceMetaDataAuthnContext => {
@@ -2379,22 +2349,22 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         },
         oidcServiceAllowDynamicRegistration => {
             type          => 'bool',
-            default       => 0,
+            default       => '0',
             documentation => 'OpenID Connect allow dynamic client registration',
         },
         oidcServiceAllowAuthorizationCodeFlow => {
             type          => 'bool',
-            default       => 1,
+            default       => '1',
             documentation => 'OpenID Connect allow authorization code flow',
         },
         oidcServiceAllowImplicitFlow => {
             type          => 'bool',
-            default       => 0,
+            default       => '0',
             documentation => 'OpenID Connect allow implicit flow',
         },
         oidcServiceAllowHybridFlow => {
             type          => 'bool',
-            default       => 0,
+            default       => '0',
             documentation => 'OpenID Connect allow hybrid flow',
         },
         oidcStorage        => { type => 'PerlModule', },
@@ -2499,8 +2469,8 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         oidcRPMetaDataOptionsRedirectUris => { type => 'text', },
         oidcRPMetaDataOptionsExtraClaims =>
           { type => 'keyTextContainer', default => {} },
-        oidcRPMetaDataOptionsBypassConsent => { type => 'bool', default => 0 },
-        oidcRPMetaDataOptionsPostLogoutRedirectUris => { type => 'text', },
+        oidcRPMetaDataOptionsBypassConsent =>
+          { type => 'bool', help => 'openidconnectclaims.html', default => 0 },
 
     };
 }
