@@ -14,6 +14,8 @@
 
 package Lemonldap::NG::Manager::Build::CTrees;
 
+our $VERSION = '1.9.99_02';
+
 sub cTrees {
     return {
         virtualHost => [
@@ -25,7 +27,8 @@ sub cTrees {
                 help  => 'configvhost.html#options',
                 nodes => [
                     'vhostPort',        'vhostHttps',
-                    'vhostMaintenance', 'vhostAliases'
+                    'vhostMaintenance', 'vhostAliases',
+                    'vhostType',
                 ],
             },
         ],
@@ -196,9 +199,9 @@ sub cTrees {
                     'oidcRPMetaDataOptionsAccessTokenExpiration',
                     'oidcRPMetaDataOptionsRedirectUris',
                     'oidcRPMetaDataOptionsBypassConsent',
-                    'oidcRPMetaDataOptionsExtraClaims'
                 ]
             },
+            'oidcRPMetaDataOptionsExtraClaims',
         ],
     };
 }
