@@ -13,7 +13,7 @@
 
 package Lemonldap::NG::Manager::Build::Tree;
 
-our $VERSION = '1.9.11';
+our $VERSION = '1.9.99_02';
 
 # TODO: Missing:
 #  * activeTimer
@@ -78,9 +78,7 @@ sub tree {
                                         'portalUserAttr',
                                         'portalOpenLinkInNewWindow',
                                         'portalAntiFrame',
-                                        'portalPingInterval',
-                                        'portalErrorOnExpiredSession',
-                                        'portalErrorOnMailNotFound'
+                                        'portalPingInterval'
                                     ]
                                 }
                             ]
@@ -93,8 +91,6 @@ sub tree {
                                 'captcha_mail_enabled',
                                 'captcha_register_enabled',
                                 'captcha_size',
-                                'captchaStorage',
-                                'captchaStorageOptions'
                             ]
                         }
                     ]
@@ -122,19 +118,6 @@ sub tree {
                             form  => 'simpleInputContainer',
                             nodes => ['apacheAuthnLevel']
                         },
-                        {
-                            title => 'browseridParams',
-                            help  => 'authbrowserid.html',
-                            form  => 'simpleInputContainer',
-                            nodes => [
-                                'browserIdAuthnLevel',
-                                'browserIdAutoLogin',
-                                'browserIdVerificationURL',
-                                'browserIdSiteName',
-                                'browserIdSiteLogo',
-                                'browserIdBackgroundColor'
-                            ]
-                        },
                         {
                             title => 'casParams',
                             help  => 'authcas.html',
@@ -191,20 +174,8 @@ sub tree {
                                 {
                                     title => 'dbiPassword',
                                     help  => 'authdbi.html#password',
-                                    nodes => [
-                                        'dbiAuthPasswordHash',
-                                        {
-                                            title => 'dbiDynamicHash',
-                                            help  => 'authdbi.html#password',
-                                            form  => 'simpleInputContainer',
-                                            nodes => [
-                                                'dbiDynamicHashEnabled',
-                                                'dbiDynamicHashValidSchemes',
-'dbiDynamicHashValidSaltedSchemes',
-'dbiDynamicHashNewPasswordScheme'
-                                            ]
-                                        }
-                                    ]
+                                    form  => 'simpleInputContainer',
+                                    nodes => ['dbiAuthPasswordHash']
                                 }
                             ]
                         },
@@ -221,12 +192,6 @@ sub tree {
                                 'facebookAppId',      'facebookAppSecret'
                             ]
                         },
-                        {
-                            title => 'googleParams',
-                            help  => 'authgoogle.html',
-                            nodes =>
-                              [ 'googleAuthnLevel', 'googleExportedVars' ]
-                        },
                         {
                             title => 'ldapParams',
                             help  => 'authldap.html',
@@ -285,19 +250,9 @@ sub tree {
                             ]
                         },
                         {
-                            title => 'linkedinParams',
-                            help  => 'authlinkedin.html',
-                            nodes => [
-                                'linkedInAuthnLevel',   'linkedInClientID',
-                                'linkedInClientSecret', 'linkedInFields',
-                                'linkedInUserField',    'linkedInScope'
-                            ]
-                        },
-                        {
-                            title => 'multiParams',
-                            help  => 'authmulti.html',
-                            form  => 'authParamsTextContainer',
-                            nodes => [ 'multiAuthStack', 'multiUserDBStack' ]
+                            title => 'combinationParams',
+                            help  => 'authcombination.html',
+                            nodes => [ 'combination', 'combModules' ]
                         },
                         {
                             title => 'nullParams',
@@ -327,8 +282,9 @@ sub tree {
                             help  => 'authproxy.html',
                             form  => 'simpleInputContainer',
                             nodes => [
-                                'soapAuthService', 'remoteCookieName',
-                                'soapSessionService'
+                                'proxyAuthService', 'proxySessionService',
+                                'remoteCookieName', 'proxyAuthnLevel',
+                                'proxyUseSoap'
                             ]
                         },
                         {
@@ -340,6 +296,15 @@ sub tree {
                                 'radiusServer'
                             ]
                         },
+                        {
+                            title => 'restParams',
+                            help  => 'authrest.html',
+                            form  => 'simpleInputContainer',
+                            nodes => [
+                                'restAuthUrl',       'restUserDBUrl',
+                                'restPwdConfirmUrl', 'restPwdModifyUrl'
+                            ]
+                        },
                         {
                             title => 'remoteParams',
                             help  => 'authremote.html',
@@ -363,7 +328,7 @@ sub tree {
                             title => 'sslParams',
                             help  => 'authssl.html',
                             form  => 'simpleInputContainer',
-                            nodes => [ 'SSLAuthnLevel', 'SSLVar' ]
+                            nodes => [ 'SSLAuthnLevel', 'SSLVar', 'SSLVarIf' ]
                         },
                         {
                             title => 'twitterParams',
@@ -469,6 +434,7 @@ sub tree {
                         },
                         {
                             title => 'issuerDBGet',
+                            help  => 'issuerdbget.html',
                             nodes => [
                                 'issuerDBGetActivation',
                                 'issuerDBGetPath',
@@ -482,10 +448,8 @@ sub tree {
                     title => 'logParams',
                     help  => 'logs.html',
                     form  => 'simpleInputContainer',
-                    nodes => [
-                        'syslog',      'trustedProxies',
-                        'whatToTrace', 'hiddenAttributes'
-                    ]
+                    nodes =>
+                      [ 'trustedProxies', 'whatToTrace', 'hiddenAttributes' ]
                 },
                 {
                     title => 'cookieParams',
@@ -535,14 +499,17 @@ sub tree {
                 },
                 'reloadUrls',
                 {
-                    title => 'advancedParams',
-                    help  => 'start.html#advanced_features',
+                    title => 'plugins',
                     nodes => [
-                        'customFunctions',
+                        'portalStatus',
                         {
-                            title => 'soap',
+                            title => 'portalServers',
                             form  => 'simpleInputContainer',
-                            nodes => [ 'Soap', 'exportedAttr' ]
+                            nodes => [
+                                'restSessionServer', 'restConfigServer',
+                                'soapSessionServer', 'soapConfigServer',
+                                'exportedAttr'
+                            ]
                         },
                         {
                             title => 'loginHistory',
@@ -559,6 +526,8 @@ sub tree {
                             help  => 'notifications.html',
                             nodes => [
                                 'notification',
+                                'notificationServer',
+                                'oldNotifFormat',
                                 'notificationStorage',
                                 'notificationStorageOptions',
                                 'notificationWildcard',
@@ -611,12 +580,27 @@ sub tree {
                             form  => 'simpleInputContainer',
                             nodes => [
                                 'registerDB',
-                                'registerUrl',
                                 'registerTimeout',
                                 'registerConfirmSubject',
                                 'registerDoneSubject'
                             ]
                         },
+                        {
+                            title => 'u2f',
+                            help  => 'u2f.html',
+                            form  => 'simpleInputContainer',
+                            nodes => [
+                                'u2fActivation', 'u2fSelfRegistration',
+                                'u2fAuthnLevel'
+                            ]
+                        },
+                    ]
+                },
+                {
+                    title => 'advancedParams',
+                    help  => 'start.html#advanced_features',
+                    nodes => [
+                        'customFunctions',
                         {
                             title => 'security',
                             help => 'security.html#configure_security_settings',
@@ -629,7 +613,20 @@ sub tree {
                                 'trustedDomains',
                                 'useSafeJail',
                                 'checkXSS',
-                                'lwpSslOpts'
+                                'lwpOpts',
+                                'lwpSslOpts',
+                                {
+                                    title => 'contentSecurityPolicy',
+                                    help  => 'csp.html',
+                                    form  => 'simpleInputContainer',
+                                    nodes => [
+                                        'cspDefault', 'cspImg',
+                                        'cspScript',  'cspStyle',
+                                        'cspConnect', 'cspFont',
+                                    ]
+                                },
+                                'requireToken',
+                                'formTimeout',
                             ]
                         },
                         {
@@ -650,42 +647,6 @@ sub tree {
                             form  => 'simpleInputContainer',
                             nodes => [ 'jsRedirect', 'noAjaxHook' ]
                         },
-                        {
-                            title => 'specialHandlers',
-                            nodes => [
-                                {
-                                    title => 'zimbraHandler',
-                                    help  => 'applications/zimbra.html',
-                                    form  => 'simpleInputContainer',
-                                    nodes => [
-                                        'zimbraPreAuthKey',
-                                        'zimbraAccountKey',
-                                        'zimbraBy',
-                                        'zimbraUrl',
-                                        'zimbraSsoUrl'
-                                    ]
-                                },
-                                {
-                                    title => 'sympaHandler',
-                                    help  => 'applications/sympa.html',
-                                    form  => 'simpleInputContainer',
-                                    nodes => [ 'sympaSecret', 'sympaMailKey' ]
-                                },
-                                {
-                                    title => 'secureTokenHandler',
-                                    help  => 'securetoken.html',
-                                    form  => 'simpleInputContainer',
-                                    nodes => [
-                                        'secureTokenMemcachedServers',
-                                        'secureTokenExpiration',
-                                        'secureTokenAttribute',
-                                        'secureTokenUrls',
-                                        'secureTokenHeader',
-                                        'secureTokenAllowOnError'
-                                    ]
-                                }
-                            ]
-                        },
                         'nginxCustomHandlers',
                         'logoutServices',
                         'multiValuesSeparator',
@@ -702,6 +663,7 @@ sub tree {
         },
         {
             title => 'variables',
+            help  => 'variables.html',
             nodes => [ 'exportedVars', 'macros', 'groups' ]
         },
         'virtualHosts',
@@ -808,7 +770,6 @@ sub tree {
 'samlIDPSSODescriptorSingleSignOnServiceHTTPRedirect',
 'samlIDPSSODescriptorSingleSignOnServiceHTTPPost',
 'samlIDPSSODescriptorSingleSignOnServiceHTTPArtifact',
-                                'samlIDPSSODescriptorSingleSignOnServiceSOAP'
                             ]
                         },
                         {
