@@ -21,10 +21,12 @@ package Lemonldap::NG::Manager::Conf::Parser;
 use strict;
 use utf8;
 use Mouse;
-use Lemonldap::NG::Manager::Constants;
+use Lemonldap::NG::Common::Conf::ReConstants;
 use Lemonldap::NG::Manager::Attributes;
 
-our $VERSION = '1.9.4';
+our $VERSION = '1.9.99_02';
+
+extends 'Lemonldap::NG::Common::Conf::Compact';
 
 # High debugging for developpers, set this to 1
 use constant HIGHDEBUG => 0;
@@ -91,17 +93,15 @@ sub hdebug {
 sub check {
     my $self = shift;
     hdebug("# check()");
-    my $res;
     unless ( $self->newConf ) {
-        $res = $self->scanTree;
-        return 0 unless ($res);
+        return 0 unless ( $self->scanTree );
     }
-
     unless ( $self->testNewConf ) {
         hdebug("  testNewConf() failed");
         return 0;
     }
     hdebug("  tests succeed");
+    $self->compactConf( $self->newConf );
     unless ( $self->confChanged ) {
         hdebug("  no changes detected");
         $self->message('__confNotChanged__');
@@ -125,7 +125,7 @@ sub scanTree {
     $self->newConf->{cfgAuthor} =
       $self->req->userData->{ $Lemonldap::NG::Handler::Main::tsv->{whatToTrace}
           || '_whatToTrace' } // "anonymous";
-    $self->newConf->{cfgAuthorIP} = $self->req->remote_ip;
+    $self->newConf->{cfgAuthorIP} = $self->req->address;
     $self->newConf->{cfgDate}     = time;
     $self->newConf->{key} ||=
       join( '', map { chr( int( rand(94) ) + 33 ) } ( 1 .. 16 ) );
@@ -316,7 +316,6 @@ sub _scanNodes {
                     }
                     elsif ($h) {
                         hdebug("  $target: opened node");
-                        $self->confChanged(1);
                         $self->set( $target, $key, $leaf->{title},
                             $leaf->{data} );
                     }
@@ -461,16 +460,14 @@ sub _scanNodes {
                         : {}
                     }
                   );
-                for ( my $i = 0 ; $i < @listCatNew; $i++ ) {
-                    if ( not( defined $listCatRef[$i] )
-                        or $listCatRef[$i] ne $listCatNew[$i] )
-                    {
+                foreach my $cat (@listCatNew) {
+                    @listCatRef = grep { $_ ne $cat } @listCatRef;
+                }
+                if (@listCatRef) {
+                    $self->confChanged(1);
+                    foreach my $cat (@listCatRef) {
                         push @{ $self->changes },
-                          {
-                            key => $leaf->{id},
-                            new => $listCatNew[$i],
-                            old => $listCatRef[$i]
-                          };
+                          { key => $leaf->{id}, old => $cat };
                     }
                 }
             }
@@ -646,6 +643,25 @@ sub _scanNodes {
             next;
         }
 
+        # combModules: just to replace "over" key
+        elsif ( $name eq 'combModules' ) {
+            hdebug('combModules');
+            $self->newConf->{$name} = {};
+            foreach my $node ( @{ $leaf->{nodes} } ) {
+                my $tmp;
+                $tmp->{$_} = $node->{data}->{$_} foreach (qw(type for));
+                $tmp->{over} = {};
+                foreach ( @{ $node->{data}->{over} } ) {
+                    $tmp->{over}->{ $_->[0] } = $_->[1];
+                }
+                $self->newConf->{$name}->{ $node->{title} } = $tmp;
+            }
+
+            # TODO: check changes
+            $self->confChanged(1);
+            next;
+        }
+
         ####################
         # Other hash nodes #
         ####################
@@ -884,7 +900,7 @@ sub set {
 
 sub defaultValue {
     my ( $self, $target ) = @_;
-    hdebug("# defaultValue($target)");
+    hdebug("# defautValue($target)");
     die unless ($target);
     my $res = eval {
         &Lemonldap::NG::Manager::Attributes::attributes()->{$target}
@@ -914,12 +930,6 @@ sub _unitTest {
     my $attrs = &Lemonldap::NG::Manager::Attributes::attributes();
     my $res   = 1;
     foreach my $key ( keys %$conf ) {
-        if (    $self->{skippedUnitTests}
-            and $self->{skippedUnitTests} =~ /\b$key\b/ )
-        {
-            $self->lmLog( "Ignore test for $key", 'debug' );
-            next;
-        }
         hdebug("Testing $key");
         my $attr = $attrs->{$key};
         my $type = $types->{ $attr->{type} };
@@ -1054,12 +1064,6 @@ sub _globalTest {
     my $result = 1;
     my $tests  = &Lemonldap::NG::Manager::Conf::Tests::tests( $self->newConf );
     foreach my $name ( keys %$tests ) {
-        if (    $self->{skippedGlobalTests}
-            and $self->{skippedGlobalTests} =~ /\b$name\b/ )
-        {
-            $self->lmLog( "Ignore test for $name", 'debug' );
-            next;
-        }
         my $sub = $tests->{$name};
         my ( $res, $msg );
         eval {
@@ -1129,14 +1133,18 @@ L<Lemonldap::NG::Manager>, L<http://lemonldap-ng.org/>
 
 =over
 
-=item Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+=item LemonLDAP::NG team L<http://lemonldap-ng.org/team>
 
 =back
 
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
+L<http://jira.ow2.org>
+
+If you want to report a bug concerning configuration upload, please change
+HIGHDEBUG constant value to produce more logs. Then post them in the Jira
+ticket.
 
 =head1 DOWNLOAD
 
@@ -1145,13 +1153,7 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =head1 COPYRIGHT AND LICENSE
 
-=over
-
-=item Copyright (C) 2015-2016 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
-
-=item Copyright (C) 2015-2016 by Clément Oudot, E<lt>clem.oudot@gmail.comE<gt>
-
-=back
+See COPYING file for details.
 
 This library is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
